import unittest
from unittest.mock import Mock, patch

from requests.models import Response

from meticulous.api import Api
from meticulous.profile import Profile
from tests.mock_responses import MOCK_PROFILE_RESPONSE


class TestProfile(unittest.TestCase):
    def setUp(self) -> None:
        self.api = Api(base_url="http://localhost:8080/")

    @patch("requests.Session.get")
    def test_get_profile(self, mock_get: Mock) -> None:
        mock_response = Mock(spec=Response)
        mock_response.status_code = 200
        mock_response.json.return_value = MOCK_PROFILE_RESPONSE
        mock_get.return_value = mock_response

        profile_id = "05051ed3-9996-43e8-9da6-963f2b31d481"
        profile: Profile = self.api.get_profile(profile_id)

        self.assertEqual(profile.name, "Italian limbus")
        self.assertEqual(profile.id, "05051ed3-9996-43e8-9da6-963f2b31d481")
        self.assertEqual(profile.author, "meticulous")
        self.assertEqual(profile.author_id, "d9123a0a-d3d7-40fd-a548-b81376e43f23")
        self.assertEqual(len(profile.previous_authors), 2)
        self.assertEqual(profile.previous_authors[0].name, "mimoja")
        self.assertEqual(
            profile.previous_authors[1].author_id,
            "ee86a777-fdd6-46d6-8cf7-099a9fb609a1",
        )
        self.assertEqual(profile.temperature, 90.5)
        self.assertEqual(profile.final_weight, 40)
        self.assertEqual(len(profile.variables), 1)
        self.assertEqual(profile.variables[0].name, "Pressure")
        self.assertEqual(profile.variables[0].key, "pressure_1")
        self.assertEqual(profile.variables[0].value, "$pressure_var")
        self.assertEqual(len(profile.stages), 2)
        self.assertEqual(profile.stages[0].name, "Preinfusion")
        self.assertEqual(
            profile.stages[0].dynamics.points,
            [[0, "${flow_start}"], [5, "$flow_mid"]],
        )
        self.assertEqual(profile.stages[0].exit_triggers[0].type, "time")
        self.assertEqual(profile.stages[0].exit_triggers[0].value, "${time_exit}")
        self.assertEqual(profile.stages[1].limits[0].type, "flow")
        self.assertEqual(profile.stages[1].limits[0].value, "$flow_limit")
        self.assertEqual(
            profile.display.image,
            "/api/v1/profile/image/ed03e12bb34fc419c5adfd7d993b50e7.png",
        )
        self.assertEqual(profile.last_changed, 1716585650.3912911)

    @patch("requests.Session.get")
    def test_get_profile_invalid_json(self, mock_get: Mock) -> None:
        # Set up the mock to return invalid JSON
        mock_response = Mock(spec=Response)
        mock_response.status_code = 200
        mock_response.json.side_effect = ValueError("Invalid JSON")
        mock_get.return_value = mock_response

        profile_id = "05051ed3-9996-43e8-9da6-963f2b31d481"
        with self.assertRaises(ValueError) as context:
            self.api.get_profile(profile_id)

        self.assertTrue("Invalid JSON" in str(context.exception))

    @patch("requests.Session.get")
    def test_get_profile_invalid_format(self, mock_get: Mock) -> None:
        # Set up the mock to return JSON not adhering to the format
        invalid_profile_response = {
            "name": "Invalid Profile",
            "id": 12345,  # ID should be a string (UUID format)
            "author": "author_name",
            # Missing required fields such as author_id, temperature, final_weight, stages
        }
        mock_response = Mock(spec=Response)
        mock_response.status_code = 200
        mock_response.json.return_value = invalid_profile_response
        mock_get.return_value = mock_response

        profile_id = "invalid-profile-id"
        with self.assertRaises(Exception) as context:
            self.api.get_profile(profile_id)

        self.assertTrue("validation error" in str(context.exception).lower())


if __name__ == "__main__":
    unittest.main()



import json
import unittest
from unittest.mock import Mock, patch

from requests.models import Response

from meticulous import Api, APIError
from meticulous.api_types import (
    BrightnessRequest,
    DeviceInfo,
    HistoryListingResponse,
    HistoryStats,
    HistoryQueryParams,
    OSStatusResponse,
    ShotRatingResponse,
    RateShotResponse,
    DefaultProfiles,
    Regions,
)
from .mock_responses import (
    MOCK_PROFILE_LIST_RESPONSE,
    MOCK_DEVICE_INFO_RESPONSE,
    MOCK_HISTORY_LISTING_RESPONSE,
    MOCK_HISTORY_STATS_RESPONSE,
    MOCK_SETTINGS_RESPONSE,
    MOCK_DEFAULT_PROFILES_RESPONSE,
    MOCK_SHOT_RATING_RESPONSE,
    MOCK_RATE_SHOT_RESPONSE,
    MOCK_OS_STATUS_RESPONSE,
    MOCK_REGIONS_RESPONSE,
    MOCK_HISTORY_DATES_RESPONSE,
    MOCK_SHOT_FILES_RESPONSE,
    MOCK_SHOT_LOG_RESPONSE,
    MOCK_SHOT_LOG_ZST,
)


class TestApi(unittest.TestCase):
    def setUp(self) -> None:
        self.api = Api(base_url="http://localhost:8080/")

    @patch("requests.Session.get")
    def test_list_profiles(self, mock_get: Mock) -> None:
        mock_response = Mock(spec=Response)
        mock_response.status_code = 200
        mock_response.json.return_value = MOCK_PROFILE_LIST_RESPONSE
        mock_get.return_value = mock_response
        profiles = self.api.list_profiles()

        self.assertEqual(len(profiles), 3)
        self.assertEqual(profiles[0].name, "Italian limbus")
        self.assertEqual(profiles[1].display.accentColor, "#FF5733")
        self.assertEqual(profiles[2].id, "06050134-0680-408e-8fe4-60e7d329b136")

    @patch("requests.Session.get")
    def test_get_profile_not_found(self, mock_get: Mock) -> None:
        # Set up the mock to return a 404 response
        mock_response = Mock(spec=Response)
        mock_response.status_code = 404
        mock_response.json.return_value = {"error": "Profile not found"}
        mock_get.return_value = mock_response

        profile_id = "nonexistent-profile-id"
        result = self.api.get_profile(profile_id)

        self.assertIsInstance(result, APIError)
        self.assertEqual(result.error, "Profile not found")

    @patch("requests.Session.get")
    def test_get_device_info(self, mock_get: Mock) -> None:
        mock_response = Mock(spec=Response)
        mock_response.status_code = 200
        mock_response.json.return_value = MOCK_DEVICE_INFO_RESPONSE
        mock_get.return_value = mock_response

        result = self.api.get_device_info()

        self.assertIsInstance(result, DeviceInfo)
        self.assertEqual(result.name, "meticulous")
        self.assertEqual(result.serial, "MT12345678")
        self.assertEqual(result.mainVoltage, 230.5)
        self.assertEqual(len(result.version_history), 4)

    @patch("requests.Session.post")
    def test_set_brightness(self, mock_post: Mock) -> None:
        mock_response = Mock(spec=Response)
        mock_response.status_code = 200
        mock_post.return_value = mock_response

        brightness_req = BrightnessRequest(brightness=80, interpolation="curve")
        result = self.api.set_brightness(brightness_req)

        self.assertIsNone(result)
        mock_post.assert_called_once()

    @patch("requests.Session.get")
    def test_get_history_short_listing(self, mock_get: Mock) -> None:
        mock_response = Mock(spec=Response)
        mock_response.status_code = 200
        mock_response.json.return_value = MOCK_HISTORY_LISTING_RESPONSE
        mock_get.return_value = mock_response

        result = self.api.get_history_short_listing()

        self.assertIsInstance(result, HistoryListingResponse)
        self.assertEqual(len(result.history), 1)
        self.assertEqual(result.history[0].name, "Morning espresso")
        self.assertEqual(result.history[0].rating, "like")

    @patch("requests.Session.post")
    def test_search_history(self, mock_post: Mock) -> None:
        mock_response = Mock(spec=Response)
        mock_response.status_code = 200
        mock_response.json.return_value = {"history": []}
        mock_post.return_value = mock_response

        query = HistoryQueryParams(
            query="espresso", max_results=10, sort="desc", order_by=["date"]
        )
        result = self.api.search_history(query)

        self.assertIsNotNone(result)
        mock_post.assert_called_once()

    @patch("requests.Session.get")
    def test_get_history_statistics(self, mock_get: Mock) -> None:
        mock_response = Mock(spec=Response)
        mock_response.status_code = 200
        mock_response.json.return_value = MOCK_HISTORY_STATS_RESPONSE
        mock_get.return_value = mock_response

        result = self.api.get_history_statistics()

        self.assertIsInstance(result, HistoryStats)
        self.assertEqual(result.totalSavedShots, 150)
        self.assertEqual(len(result.byProfile), 3)
        self.assertEqual(result.byProfile[0].name, "Italian limbus")
        self.assertEqual(result.byProfile[0].count, 75)

    @patch("requests.Session.get")
    def test_get_settings(self, mock_get: Mock) -> None:
        mock_response = Mock(spec=Response)
        mock_response.status_code = 200
        mock_response.json.return_value = MOCK_SETTINGS_RESPONSE
        mock_get.return_value = mock_response

        result = self.api.get_settings()

        self.assertIsNotNone(result)
        self.assertEqual(result.auto_preheat, 5)
        self.assertTrue(result.enable_sounds)
        self.assertEqual(result.update_channel, "stable")

    @patch("requests.Session.get")
    def test_get_default_profiles(self, mock_get: Mock) -> None:
        mock_response = Mock(spec=Response)
        mock_response.status_code = 200
        mock_response.json.return_value = MOCK_DEFAULT_PROFILES_RESPONSE
        mock_get.return_value = mock_response

        result = self.api.get_default_profiles()

        self.assertIsInstance(result, DefaultProfiles)
        self.assertEqual(len(result.default), 1)
        self.assertEqual(len(result.community), 1)

    @patch("requests.Session.post")
    def test_rate_shot(self, mock_post: Mock) -> None:
        mock_response = Mock(spec=Response)
        mock_response.status_code = 200
        mock_response.json.return_value = MOCK_RATE_SHOT_RESPONSE
        mock_post.return_value = mock_response

        result = self.api.rate_shot(1, "like")

        self.assertIsInstance(result, RateShotResponse)
        self.assertEqual(result.shot_id, 1)
        self.assertEqual(result.rating, "like")
        self.assertEqual(result.status, "success")

    @patch("requests.Session.get")
    def test_get_shot_rating(self, mock_get: Mock) -> None:
        mock_response = Mock(spec=Response)
        mock_response.status_code = 200
        mock_response.json.return_value = MOCK_SHOT_RATING_RESPONSE
        mock_get.return_value = mock_response

        result = self.api.get_shot_rating(1)

        self.assertIsInstance(result, ShotRatingResponse)
        self.assertEqual(result.shot_id, 1)
        self.assertEqual(result.rating, "like")

    @patch("requests.Session.get")
    def test_get_os_status(self, mock_get: Mock) -> None:
        mock_response = Mock(spec=Response)
        mock_response.status_code = 200
        mock_response.json.return_value = MOCK_OS_STATUS_RESPONSE
        mock_get.return_value = mock_response

        result = self.api.get_os_status()

        self.assertIsInstance(result, OSStatusResponse)
        self.assertEqual(result.progress, 75)
        self.assertEqual(result.status, "updating")

    @patch("requests.Session.get")
    def test_get_timezone_region(self, mock_get: Mock) -> None:
        mock_response = Mock(spec=Response)
        mock_response.status_code = 200
        mock_response.json.return_value = MOCK_REGIONS_RESPONSE
        mock_get.return_value = mock_response

        result = self.api.get_timezone_region("countries", "North America")

        self.assertIsInstance(result, Regions)
        self.assertEqual(len(result.countries), 3)
        self.assertIn("US", result.countries)

    @patch("requests.Session.get")
    def test_get_current_shot_none(self, mock_get: Mock) -> None:
        mock_response = Mock(spec=Response)
        mock_response.status_code = 200
        mock_response.json.return_value = None
        mock_get.return_value = mock_response

        result = self.api.get_current_shot()

        self.assertIsNone(result)

    @patch("requests.Session.get")
    def test_get_history_dates(self, mock_get: Mock) -> None:
        mock_response = Mock(spec=Response)
        mock_response.status_code = 200
        mock_response.json.return_value = MOCK_HISTORY_DATES_RESPONSE
        mock_get.return_value = mock_response

        result = self.api.get_history_dates()

        self.assertEqual(len(result), 2)
        self.assertEqual(result[0].name, "2024-01-02")
        self.assertEqual(result[0].url, "2024-01-02")

    @patch("requests.Session.get")
    def test_get_shot_files(self, mock_get: Mock) -> None:
        mock_response = Mock(spec=Response)
        mock_response.status_code = 200
        mock_response.json.return_value = MOCK_SHOT_FILES_RESPONSE
        mock_get.return_value = mock_response

        result = self.api.get_shot_files("2024-01-02")

        self.assertEqual(len(result), 2)
        self.assertEqual(result[0].url, "21:04:06.shot.json")

    @patch("requests.Session.get")
    def test_get_shot_log_plain_json(self, mock_get: Mock) -> None:
        mock_response = Mock(spec=Response)
        mock_response.status_code = 200
        mock_response.content = json.dumps(MOCK_SHOT_LOG_RESPONSE).encode()
        mock_get.return_value = mock_response

        result = self.api.get_shot_log("2024-01-02", "21:04:06.shot.json")

        self.assertEqual(result["shot"], "latest")
        self.assertEqual(result["value"], 1)

    @patch("requests.Session.get")
    def test_get_shot_log_zst(self, mock_get: Mock) -> None:
        mock_response = Mock(spec=Response)
        mock_response.status_code = 200
        mock_response.content = MOCK_SHOT_LOG_ZST
        mock_get.return_value = mock_response

        result = self.api.get_shot_log("2024-01-02", "20:55:00.shot.json.zst")

        self.assertEqual(result["shot"], "latest")

    @patch("requests.Session.get")
    def test_get_shot_log_error(self, mock_get: Mock) -> None:
        mock_response = Mock(spec=Response)
        mock_response.status_code = 404
        mock_response.json.return_value = {"error": "not found", "status": "404"}
        mock_get.return_value = mock_response

        result = self.api.get_shot_log("2024-01-02", "missing.shot.json")

        self.assertIsInstance(result, APIError)
        self.assertEqual(result.error, "not found")

    @patch("requests.Session.get")
    def test_get_last_shot_log(self, mock_get: Mock) -> None:
        dates_response = Mock(spec=Response)
        dates_response.status_code = 200
        dates_response.json.return_value = MOCK_HISTORY_DATES_RESPONSE

        files_response = Mock(spec=Response)
        files_response.status_code = 200
        files_response.json.return_value = MOCK_SHOT_FILES_RESPONSE

        log_response = Mock(spec=Response)
        log_response.status_code = 200
        log_response.content = json.dumps(MOCK_SHOT_LOG_RESPONSE).encode()

        mock_get.side_effect = [dates_response, files_response, log_response]

        result = self.api.get_last_shot_log()

        self.assertEqual(result["shot"], "latest")


if __name__ == "__main__":
    unittest.main()
